<?php
namespace Teknisa\FIS\Controllers;

use Teknisa\FIS\Services\Fis00800InventarioFiscalService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataSet;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;


class Fis00800InventarioFiscalController extends \Teknisa\FIS\Controllers\TeknisaCrud{

  protected $dataSourceName;
  protected $parameterBag;
  protected $crudService;
  protected $service;

  public function __construct($crudService, ParameterBag $parameterBag, Fis00800InventarioFiscalService $service) {
    $this->service = $service;
    parent::__construct($crudService, $parameterBag);
  }

  public function saveInventarioFiscal(DTO\Request\DataSet $request, DTO\Response $response) {
    $this->dataSourceName = 'inventfisc_invfisc';

    $row = $request->getDataSet()->getRows()[0];

    try {

      $this->service->validateInventarioFiscal($row);

      if ($row['__is_new']) {
        $row['NRSEQINVFISC'] = $this->service->generateNrSeq('INVENTFISC');
        $newDataSet = new DataSet($this->dataSourceName, array($row));
        $newRequest = new RequestDataSet($newDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        parent::save($newRequest, $response);

      }
      else {
        parent::save($request, $response);
      }

    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), $e->getCode()));
    }

  }

  public function importInventarioFiscal(DTO\Request\Row $request, DTO\Response $response) {
    $this->dataSourceName = 'inventfisc_invfisc';

    $row = $request->getRow();

    try {
      $result = $this->service->importInventarioFiscal($row);
      $response->addDataSet($result);
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), $e->getCode()));
    }


  }




}