<?php
namespace Teknisa\FIS\Modules\Acessorias\Services;

//require("../Utilities/Agendamento.php");
use Teknisa\FIS\Modules\Acessorias\Factories\Fis85000SpedFactory;
use Teknisa\FIS\Modules\Acessorias\Services\Fis91000ObrigacoesAcessoriasService;
use Teknisa\FIS\Modules\Acessorias\Services\Agendamento;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use \Teknisa\Libs\Util\Libraries\PHPMailer;
use Zeedhi\Framework\Remote\FileUploader;
//use Zeedhi\FileServerAPI\API;

class Fis85000SpedService {

    protected $environment;
    protected $spedFactory;
    protected $obrigacoesService;
    protected $newCode;

    private $_fileServerUrl;
    protected $fileUploader; 
    
    public $mensagemBody = '';
    public $mensagemBodySucess = '';
    public $mensagemError = '';
    public $mensagemErrorBody = '';
    public $geraArquivo = '';
    
    public function __construct(Environment $environment, Fis85000SpedFactory $spedFactory, Fis91000ObrigacoesAcessoriasService $obrigacoesService, NewCode $newCode,
                                Agendamento $agendamento, $fileServerUrl, $fileUploader
    ) {
        $this->environment                  = $environment;
        $this->spedFactory                  = $spedFactory;
        $this->obrigacoesService            = $obrigacoesService;
        $this->newCode                      = $newCode;
        $this->agendamento                  = $agendamento;
        $this->_fileServerUrl               = $fileServerUrl;
        $this->fileUploader                 = $fileUploader;
        
        set_time_limit(0);
    }

    /*
     * Função para criar o arquivo da exportação SPED.
     */
    private function createFileSpedFiscal($row) {
        
        $spedData = [];
        // Verifica se há uma extensão já informada.
        $i = 0;
        $params = array(
            "NRORG"         => $row["NRORG"],
            "NRSEQARQMAG"   => $row["NRSEQARQMAG"],
            "INICIO"        => (5000*$i +1),
            "FIM"           => (5000*($i+1))
        );
        
        $spedData = $this->spedFactory->getSpedFiscalDataPeriod($params);
        
        if(!empty($spedData)){
         
            if(empty(pathinfo($row["NMARQUIVO"], PATHINFO_EXTENSION)) || substr($row["NMARQUIVO"], -1) == '.') {
                // Se o nome do arquivo for terminado em '.', termina a extensão '.txt'.
                if (substr($row["NMARQUIVO"], -1) == '.') {
                    $row["NMARQUIVO"] .= "txt";
                }
                // Senão, adiciona a extensão '.txt' ao nome do arquivo.
                else {
                    $row["NMARQUIVO"] .= ".txt";
                }
            }
    
            // Cria ou sobrepõe um arquivo com o nome dado.
            try {
                $file = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $row["NMARQUIVO"], "w");
                
            } catch (\Exception $e) {
                // Se houver algum erro ao tentar abrir o arquivo, lança uma exceção.
                throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
            }
            // Se houver algum erro após tentar abrir o arquivo, lança uma exceção.
            if (!$file) {
                throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
            }
            
            while(!empty($spedData)){
                
                $text = "";
                // Percorre os dados, concatenando na variável a ser impressa.
                foreach ($spedData as $dataRow) {
                    $text .= $dataRow["DSREGARQMAG"] . "\r\n";
                }
                
                // Imprime o texto no arquivo.
                $text = mb_convert_encoding($text, "UTF-8", "UTF-8"); 
                         
                fprintf($file, "%s", $text);
                
                $i = $i + 1;
                $params = array(
                    "NRORG" => $row["NRORG"],
                    "NRSEQARQMAG" => $row["NRSEQARQMAG"],
                    "INICIO" => (5000*$i +1),
                    "FIM" => (5000*($i+1))
                );
                
                $spedData = [];
                
                $spedData = $this->spedFactory->getSpedFiscalDataPeriod($params);
                
            }
            
            fclose($file);
            
            return $row["NMARQUIVO"];
            
        } else {
            if(isset($row['NRAGENDAMENTO'])){
                $this->mensagemError = "Não há dados a serem exibidos.";
                return $fileName = false;
            } else {
                throw new \Exception("Não há dados a serem exibidos.");
            }
        }
    }
    
    private function createFileSpedContrib($row) {
        
        $spedData = [];
        // Verifica se há uma extensão já informada.
        $i = 0;
        $params = array(
            "NRORG"         => $row["NRORG"],
            "NRSEQARQMAG"   => $row["NRSEQARQMAG"],
            "INICIO"        => (5000*$i +1),
            "FIM"           => (5000*($i+1))
        );
        
        $spedData = $this->spedFactory->getSpedContribuicoesPeriod($params);
        
        if(!empty($spedData)){
         
            if(empty(pathinfo($row["NMARQUIVO"], PATHINFO_EXTENSION)) || substr($row["NMARQUIVO"], -1) == '.') {
                // Se o nome do arquivo for terminado em '.', termina a extensão '.txt'.
                if (substr($row["NMARQUIVO"], -1) == '.') {
                    $row["NMARQUIVO"] .= "txt";
                }
                // Senão, adiciona a extensão '.txt' ao nome do arquivo.
                else {
                    $row["NMARQUIVO"] .= ".txt";
                }
            }
    
            // Cria ou sobrepõe um arquivo com o nome dado.
            try {
                $file = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $row["NMARQUIVO"], "w");
                
            } catch (\Exception $e) {
                // Se houver algum erro ao tentar abrir o arquivo, lança uma exceção.
                throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
            }
            // Se houver algum erro após tentar abrir o arquivo, lança uma exceção.
            if (!$file) {
                throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
            }
            
            while(!empty($spedData)){
                
                $text = "";
                // Percorre os dados, concatenando na variável a ser impressa.
                foreach ($spedData as $dataRow) {
                    $text .= $dataRow["DSREGARQMAG"] . "\r\n";
                }
                
                // Imprime o texto no arquivo.
                $text = mb_convert_encoding($text, "UTF-8", "UTF-8");
                         
                fprintf($file, "%s", $text);
                
                $i = $i + 1;
                $params = array(
                    "NRORG" => $row["NRORG"],
                    "NRSEQARQMAG" => $row["NRSEQARQMAG"],
                    "INICIO" => (5000*$i +1),
                    "FIM" => (5000*($i+1))
                );
                
                $spedData = [];
                
                $spedData = $this->spedFactory->getSpedContribuicoesPeriod($params);
                
            }
            
            fclose($file);
            
            return $row["NMARQUIVO"];
        } else {
            if(isset($row['NRAGENDAMENTO'])){
                $this->mensagemError = "Não há dados a serem exibidos.";
                return $fileName = false;
            } else {
                throw new \Exception("Não há dados a serem exibidos.");
            }
        }
    }

    /*private function createFileSpedFiscal($row) {
        
        // Verifica se há uma extensão já informada.
        if(empty(pathinfo($row["NMARQUIVO"], PATHINFO_EXTENSION)) || substr($row["NMARQUIVO"], -1) == '.') {
            // Se o nome do arquivo for terminado em '.', termina a extensão '.txt'.
            if (substr($row["NMARQUIVO"], -1) == '.') {
                $row["NMARQUIVO"] .= "txt";
            }
            // Senão, adiciona a extensão '.txt' ao nome do arquivo.
            else {
                $row["NMARQUIVO"] .= ".txt";
            }
        }

        // Cria ou sobrepõe um arquivo com o nome dado.
        try {
            $file = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $row["NMARQUIVO"], "w");
        } catch (\Exception $e) {
            // Se houver algum erro ao tentar abrir o arquivo, lança uma exceção.
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }
        // Se houver algum erro após tentar abrir o arquivo, lança uma exceção.
        if (!$file) {
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }
        
        $spedData = [];
        $i = 0;
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRSEQARQMAG" => $row["NRSEQARQMAG"],
            "INICIO" => (5000*$i +1),
            "FIM" => (5000*($i+1))
        );
            
        while(!empty($this->spedFactory->getSpedFiscalDataPeriod($params))){
            $spedData = [];
            
            $spedData = $this->spedFactory->getSpedFiscalDataPeriod($params);
            
            
            $text = "";
            // Percorre os dados, concatenando na variável a ser impressa.
            foreach ($spedData as $dataRow) {
                $text .= $dataRow["DSREGARQMAG"] . "\r\n";
            }
            
            // Imprime o texto no arquivo.
            $text = mb_convert_encoding($text, "UTF-8", "UTF-8");
                     
            fprintf($file, "%s", $text);
            
            $i = $i + 1;
            $params = array(
                "NRORG" => $row["NRORG"],
                "NRSEQARQMAG" => $row["NRSEQARQMAG"],
                "INICIO" => (5000*$i +1),
                "FIM" => (5000*($i+1))
            );
        }
        
        fclose($file);
        
        return $row["NMARQUIVO"];
    }
    
    private function createFileSpedContrib($row) {
        // Verifica se há uma extensão já informada.
        if(empty(pathinfo($row["NMARQUIVO"], PATHINFO_EXTENSION)) || substr($row["NMARQUIVO"], -1) == '.') {
            // Se o nome do arquivo for terminado em '.', termina a extensão '.txt'.
            if (substr($row["NMARQUIVO"], -1) == '.') {
                $row["NMARQUIVO"] .= "txt";
            }
            // Senão, adiciona a extensão '.txt' ao nome do arquivo.
            else {
                $row["NMARQUIVO"] .= ".txt";
            }
        }

        // Cria ou sobrepõe um arquivo com o nome dado.
        try {
            $file = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $row["NMARQUIVO"], "w");
        } catch (\Exception $e) {
            // Se houver algum erro ao tentar abrir o arquivo, lança uma exceção.
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }
        // Se houver algum erro após tentar abrir o arquivo, lança uma exceção.
        if (!$file) {
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }
        
        $spedData = [];
        $i = 0;
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRSEQARQMAG" => $row["NRSEQARQMAG"],
            "INICIO" => (5000*$i +1),
            "FIM" => (5000*($i+1))
        );
            
        while(!empty($this->spedFactory->getSpedContribuicoesPeriod($params))){
            $spedData = [];
            
            $spedData = $this->spedFactory->getSpedContribuicoesPeriod($params);
            
            
            $text = "";
            // Percorre os dados, concatenando na variável a ser impressa.
            foreach ($spedData as $dataRow) {
                $text .= $dataRow["DSREGARQMAG"] . "\r\n";
            }
            
            // Imprime o texto no arquivo.
            $text = mb_convert_encoding($text, "UTF-8", "UTF-8");
                     
            fprintf($file, "%s", $text);
            
            $i = $i + 1;
            $params = array(
                "NRORG" => $row["NRORG"],
                "NRSEQARQMAG" => $row["NRSEQARQMAG"],
                "INICIO" => (5000*$i +1),
                "FIM" => (5000*($i+1))
            );
        }
        
        fclose($file);
        
        return $row["NMARQUIVO"];
    }*/

    /*
     * Função para redirecionar a exportação SPED para o método apropriado.
     */
    public function exportarSped($row) {
        // Valida os dados enviados na requisição.
        $this->validateSped($row);

        // Se o arquivo for "ICMS", redireciona para o método de exportação Fiscal.
        if ($row["CDARQUIVO"] == 'ICMS') {
            $fileName = $this->exportarSpedFiscal($row);
        }
        // Se o arquivo for "ICMS", redireciona para o método de exportação Contribuições.
        else if ($row["CDARQUIVO"] == 'CONT') {
            $fileName = $this->exportarSpedContribuicoes($row);
        }
        // Caso não seja nenhum destes, informa que o arquivo escolhido é inválido.
        else {
            throw new \Exception("Arquivo inválido.");
        }

        // Retorna o nome do arquivo com suas possíveis modificações.
        return $fileName;
        
    }

    /*
     * Função para a exportação SPED Contribuições.
     */
    public function exportarSpedContribuicoes($row) {
        // Busca e valida os parâmetros associados.
        $parametros = $this->getParametersSpedContribuicoes($row);
        
        if(isset($row['NRAGENDAMENTO']) && $this->geraArquivo == false){
            return false;
        }
        
        // Gera o próximo código sequencial do arquivo magnético.
        $row["NRSEQARQMAG"] = $this->obrigacoesService->getNextNrseqarqmag($row["NRORG"]);
          //  var_dump($row);die;
        // Monta o array com os parâmetros para chamar a procedure.
        $params = array(
            "CDLIVROMAG"    => $row["CDLIVROMAG"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "DTINICIAL"     => $row["DTINICIAL"],
            "DTFINAL"       => $row["DTFINAL"],
            "VERSAO"        => $parametros["STVERSAO"],
            "INDESCRITU"    => null,
            "IDSITUESP"     => $parametros["STIDSITESP"],
            "NUMRECANT"     => $parametros["STRECANT"],
            "ATIVIDADE"     => $parametros["IDATIV"],
            "CDINCTRIB"     => $parametros["STINCTRIB"],
            "INDAPROCRED"   => $parametros["STAPROCRED"],
            "CDTPCON"       => $parametros["STTPCONTR"],
            "TPESCRITU"     => $parametros["STFINALIDADE"],
            "NRSEQARQMAG"   => $row["NRSEQARQMAG"],
            "GERABCLP"      => $parametros["STBLOCOP"],
            "CRTESCRITUR"   => $parametros["STCRTESCRIT"],
            "NATUJURID"     => $parametros["STNATUJUR"],
            "FORAPRAZO"     => $parametros["IDFORAPER"],
            "TPNATJURPJ"    => null,
            "LIVROMAG"      => null
        );

        // Chama a procedure de exportação.
        $this->spedFactory->callSpedContribuicoesProcedure($params);

        // Foi evitado utilizar um bloco finally devido a possíveis
        // incompatibilidades com a versão do PHP.
        try{
            // Seta os parâmetros para exclusão dos dados temporários.
            $params = array(
                "NRORG" => $row["NRORG"],
                "NRSEQARQMAG" => $row["NRSEQARQMAG"]
            );
            // Chama a função para gerar o arquivo texto.
            $fileName = $this->createFileSpedContrib($row);

            // Exclui os dados temporários da exportação no banco de dados.
            $this->spedFactory->deleteSpedContribuicoesData($params);

            // Retorna o nome do arquivo com suas possíveis modificações.
            return $fileName;
        } catch(\Exception $e) {
            // Exclui os dados temporários da exportação no banco de dados.
            $this->spedFactory->deleteSpedContribuicoesData($params);
            throw $e;
        }
    }

    /*
     * Função para a exportação SPED Fiscal.
     */
    public function exportarSpedFiscal($row) {
        // Busca e valida os parâmetros associados.
        $parametros = $this->getParametersSpedFiscal($row);
        
        if($parametros){
            // Gera o código sequencial do arquivo magnético através do NOVOCODIGO.
            $row["NRSEQARQMAG"] = $this->obrigacoesService->getNextNrseqarqmag($row["NRORG"]);
            // Monta o array com os parâmetros para chamar a procedure.
            $params = array(
                "CDIMPOSTO"         => "00",
                "CDEMPRESA"         => $row["CDEMPRESA"],
                "CDINSCESTA"        => $row["CDINSCESTA"],
                "CDINSCMUNI"        => isset($row["CDINSCMUNI"]) ? $row["CDINSCMUNI"] : 'N',
                "DTINICIAL"         => $row["DTINICIAL"],
                "DTFINAL"           => $row["DTFINAL"],
                "LIVROMAG"          => $row["CDLIVROMAG"],
                "FINALIDADE"        => $parametros["IDFINARQUIVO"],
                "PERFIL"            => $parametros["IDPERAPRESTACO"],
                "ATIVIDADE"         => $parametros["IDINDATIVIDADE"],
                "VERSAO"            => $parametros["CDVERSAO"],
                "NRSEQARQMAG"       => $row["NRSEQARQMAG"],
                "BLOCOH"            => $row["IDGERABLOCOH"],
                "H020"              => $row["H020"],
                "DTINIPER"          => $row["DTINIPERFISC"],
                "DTFINPER"          => $row["DTFINPERFISC"],
                "IDDIFALIQ"         => $parametros["IDGERDIFALIQ"],
                "GERAFORSALE"       => 'N',
                "MOTINV"            => $row["CDMOTINVENTARIO"],
                "OBRIGREGBL1"       => $parametros["CDOBRIGAREGBL1"],
                "CDLIVROMAGADIC"    => $row["CDLIVROMAGADIC"],
                "IDORIGLIVROADIC"   => 'L',
                "DTINIVAF"          => $row["DTINIVAF"],
                "DTFINVAF"          => $row["DTFINVAF"],
                "GERABLOCOK"        => $parametros["IDGERABLOCOK"],
                "ORIGBLOCOK"        => $parametros["IDORIGBLOCOK"],
                "TIPO"              => null,
                "INDESCRITU"        => null,
                "AGRUPPROD"         => isset($row["CDGRUPPROD"]) ? $row["CDGRUPPROD"] : null,
                "CDINSCENTR"        => isset($row["CDINSCENTR"]) ? $row["CDINSCENTR"] : 'N',
                
                "IDCLASSIFICESTAB"  => $parametros["IDCLASSIFICESTAB"],
                "IDINFCOMPLST"      => $parametros["IDINFCOMPLST"],
                "NRDIAMEDIAEST"     => $parametros["NRDIAMEDIAEST"],
                "IDGERACAMPO11ST"   => $parametros["IDGERACAMPO11ST"],
                "IDCONSPRODEST"     => $parametros["IDCONSPRODEST"]
            );
    
            // Chama a procedure de exportação.
            $this->spedFactory->callSpedFiscalProcedure($params);
    
            // Foi evitado utilizar um bloco finally devido a possíveis
            // incompatibilidades com a versão do PHP.
            try{
                // Seta os parâmetros para exclusão dos dados temporários.
                $params = array(
                    "NRORG" => $row["NRORG"],
                    "NRSEQARQMAG" => $row["NRSEQARQMAG"]
                );
                // Chama a função para gerar o arquivo texto.
                $fileName = $this->createFileSpedFiscal($row);
    
                // Exclui os dados temporários da exportação no banco de dados.
                $this->spedFactory->deleteSpedFiscalData($params);
                // Retorna o nome do arquivo com suas possíveis modificações.
                return $fileName;
            } catch(\Exception $e) {
                // Exclui os dados temporários da exportação no banco de dados.
                $this->spedFactory->deleteSpedFiscalData($params);
                throw $e;
            }
        }
    }

    /*
     * Função para buscar e validar os parâmetros da empresa para poder chamar a procedure.
     */
    private function getParametersSpedContribuicoes($row) {
        $params = array(
            "CDEMPRESA" => $row["CDEMPRESA"],
            "NRORG" => $row["NRORG"]
        );
        
        // Busca os parâmetros conforme empresa e nrorg.
        $parametros = $this->spedFactory->getParametrosSpedContribuicoes($params);

        if(isset($row['NRAGENDAMENTO'])) {
            
            $this->mensagemError = ''    ;
            
            // Verifica se há algum parâmetro cadastrado.
            if ($parametros == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Parâmetros não cadastrados para empresa selecionada.';
                $this->geraArquivo = false;
                
                return $parametros;
                //throw new \Exception("Parâmetros não cadastrados para a empresa selecionada.");
            }
            // Verifica se o campo "Finalidade do Arquivo" possui algum valor informado.
            if ($parametros["STFINALIDADE"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Finalidade do Arquivo deve ser informada.");
            }
            // Verifica se o campo "Indicador de Atividade" possui algum valor informado.
            if ($parametros["IDATIV"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Indicador de Atividade deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Indicador de Atividade deve ser informado.");
            }
            // Verifica se o campo "Incidência Tributária" possui algum valor informado.
            if ($parametros["STINCTRIB"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Incidência Tributária deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Incidência Tributária deve ser informada.");
            }
            // Verifica se o campo "Tipo de Contribuição" possui algum valor informado.
            if ($parametros["STTPCONTR"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Tipo de Contribuição deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Tipo de Contribuição deve ser informado.");
            }
            // Verifica se o campo "Critério de Escrituração" possui algum valor informado.
            // Somente se o campo "Incidência Tributária" estiver com o valor "2".
            if ($parametros["STCRTESCRIT"] == null && $parametros["STINCTRIB"] == "2") {
                $this->mensagemError = $this->mensagemError.'<br />'.'Critério de Escrituração deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Critério de Escrituração deve ser informado.");
            }
            // Verifica se o campo "Natureza Jurídica" possui algum valor informado.
            if ($parametros["STNATUJUR"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Natureza Jurídica deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Natureza Jurídica deve ser informada.");
            }
            if ($parametros["IDFORAPER"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Entrega Fora do Prazo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Entrega Fora do Prazo deve ser informada.");
            }
            // Verifica se o campo "Versão" possui algum valor informado.
            if ($parametros["STVERSAO"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Versão deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Versão deve ser informado.");
            }
            
        } else {
            // Verifica se há algum parâmetro cadastrado.
            if ($parametros == null) {
                throw new \Exception("Parâmetros não cadastrados para a empresa selecionada.");
            }
            // Verifica se o campo "Finalidade do Arquivo" possui algum valor informado.
            if ($parametros["STFINALIDADE"] == null) {
                throw new \Exception("Finalidade do Arquivo deve ser informada.");
            }
            // Verifica se o campo "Indicador de Atividade" possui algum valor informado.
            if ($parametros["IDATIV"] == null) {
                throw new \Exception("Indicador de Atividade deve ser informado.");
            }
            // Verifica se o campo "Incidência Tributária" possui algum valor informado.
            if ($parametros["STINCTRIB"] == null) {
                throw new \Exception("Incidência Tributária deve ser informada.");
            }
            // Verifica se o campo "Tipo de Contribuição" possui algum valor informado.
            if ($parametros["STTPCONTR"] == null) {
                throw new \Exception("Tipo de Contribuição deve ser informado.");
            }
            // Verifica se o campo "Critério de Escrituração" possui algum valor informado.
            // Somente se o campo "Incidência Tributária" estiver com o valor "2".
            if ($parametros["STCRTESCRIT"] == null && $parametros["STINCTRIB"] == "2") {
                throw new \Exception("Critério de Escrituração deve ser informado.");
            }
            // Verifica se o campo "Natureza Jurídica" possui algum valor informado.
            if ($parametros["STNATUJUR"] == null) {
                throw new \Exception("Natureza Jurídica deve ser informada.");
            }
             // Verifica se o campo "Entrega Fora do Prazo" possui algum valor informado.
            if ($parametros["IDFORAPER"] == null) {
                throw new \Exception("Entrega Fora do Prazo deve ser informada.");
            }
            // Verifica se o campo "Versão" possui algum valor informado.
            if ($parametros["STVERSAO"] == null) {
                throw new \Exception("Versão deve ser informado.");
            }
        }
        return $parametros;
    }

    /*
     * Função para buscar e validar os parâmetros da empresa e inscrição estadual
     * para poder chamar a procedure.
     */
    private function getParametersSpedFiscal($row) {
        $params = array(
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "NRORG"         => $row["NRORG"]
        );
        // Busca os parâmetros conforme empresa e nrorg.
        $parametros = $this->spedFactory->getParametrosSpedFiscal($params);
        
        if(isset($row['NRAGENDAMENTO'])) {
            $this->mensagemError = '';
            if ($parametros == null) {
                //Mudar dataSource para pegar a empresa
                $this->mensagemError = $this->mensagemError.'<br />'.'Parâmetros não cadastrados para empresa e inscrição estadual selecionadas.';
                return $parametros;
                //throw new \Exception("Parâmetros não cadastrados para empresa e inscrição estadual selecionadas.");
                
            }
            // Verifica se o campo "Finalidade do Arquivo" possui algum valor informado.
            if ($parametros["IDFINARQUIVO"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                //return $parametros;
                //throw new \Exception("Finalidade do Arquivo deve ser informada.");
                
            }
            // Verifica se o campo "Perfil de Apresentação" possui algum valor informado.
            if ($parametros["IDPERAPRESTACO"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Perfil de Apresentação deve ser informado.';
                //return $parametros;
                
                //throw new \Exception("Perfil de Apresentação deve ser informado.");
            }
            // Verifica se o campo "Indicador de Atividade" possui algum valor informado.
            if ($parametros["IDINDATIVIDADE"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Indicador de Atividade deve ser informado.';
                //return $parametros;
                //throw new \Exception("Indicador de Atividade deve ser informado.");
            }
            // Verifica se o campo "Gera Bloco K" possui algum valor informado.
            if ($parametros["IDGERABLOCOK"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Gera Bloco K deve ser informado.';
                //return $parametros;
                //throw new \Exception("Gera Bloco K deve ser informado.");
            }
            // Verifica se o campo "Origem Dados (Bloco K)" possui algum valor informado.
            // Somente se o campo "Gera Bloco K" estiver com o valor "S".
            if ($parametros["IDORIGBLOCOK"] == null && $parametros["IDGERABLOCOK"] == 'S') {
                $this->mensagemError = $this->mensagemError.'<br />'.'Origem Dados (Bloco K) deve ser informado.';
                //return $parametros;
                //throw new \Exception("Origem Dados (Bloco K) deve ser informado.");
            }
            // Verifica se o campo "Gerar Diferença Alíquota" possui algum valor informado.
            if ($parametros["IDGERDIFALIQ"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Gerar Diferença Alíquota deve ser informado.'; 
                //return $parametros;
                //throw new \Exception("Gerar Diferença Alíquota deve ser informado.");
            }
            // Verifica se o campo "Versão" possui algum valor informado.
            if ($parametros["CDVERSAO"] == null) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Versão deve ser informada.';
                //return $parametros;
                //throw new \Exception("Versão deve ser informada.");
            }
            // Validações adicionais caso o registro 1400 do campo
            // "Obrigatoriedade de Registros - Bloco 1" esteja marcado.
            if (substr($parametros["CDOBRIGAREGBL1"], 4, 1) == 'S') {
                // Busca o estado da inscrição estadual relacionada.
                $sgestado = $this->spedFactory->getEstadoInscricaoEstadual($params);
    
                // Validações para a tela exibida no caso do registro 1400 estar
                // marcado, somente para os estados "ES", "MG" e "SP".
                if ($sgestado == "ES" || $sgestado == "MG" || $sgestado == "SP") {
                    // Verifica se o campo "Livro Magnético" possui algum valor informado.
                    if ($row["CDLIVROMAGADIC"] == null) {
                        $this->mensagemError = $this->mensagemError.'<br />'.'Livro Magnético deve ser informado.';
                        //return $parametros;
                        //throw new \Exception("Livro Magnético deve ser informado.");
                    }
                    // Verifica se o campo "Data Inicial" possui algum valor informado.
                    // Obrigatório somente para o estado "MG".
                    if ($row["DTINIVAF"] == null && $sgestado == "MG") {
                        $this->mensagemError = $this->mensagemError.'<br />'.'Data Inicial deve ser informada.';
                        //return $parametros;
                        //throw new \Exception("Data Inicial deve ser informada.");
                    }
                    // Verifica se o campo "Data Final" possui algum valor informado.
                    // Obrigatório somente para o estado "MG".
                    if ($row["DTFINVAF"] == null && $sgestado == "MG") {
                        $this->mensagemError = $this->mensagemError.'<br />'.'Data Final deve ser informada.';
                        //return $parametros;
                        //throw new \Exception("Data Final deve ser informada.");
                    }
                }
            }
        }
        else {
            // Verifica se há algum parâmetro cadastrado.
            if ($parametros == null) {
                throw new \Exception("Parâmetros não cadastrados para empresa e inscrição estadual selecionadas.");
            }
            // Verifica se o campo "Finalidade do Arquivo" possui algum valor informado.
            if ($parametros["IDFINARQUIVO"] == null) {
                throw new \Exception("Finalidade do Arquivo deve ser informada.");
            }
            // Verifica se o campo "Perfil de Apresentação" possui algum valor informado.
            if ($parametros["IDPERAPRESTACO"] == null) {
                throw new \Exception("Perfil de Apresentação deve ser informado.");
            }
            // Verifica se o campo "Indicador de Atividade" possui algum valor informado.
            if ($parametros["IDINDATIVIDADE"] == null) {
                throw new \Exception("Indicador de Atividade deve ser informado.");
            }
            // Verifica se o campo "Gera Bloco K" possui algum valor informado.
            if ($parametros["IDGERABLOCOK"] == null) {
                throw new \Exception("Gera Bloco K deve ser informado.");
            }
            // Verifica se o campo "Origem Dados (Bloco K)" possui algum valor informado.
            // Somente se o campo "Gera Bloco K" estiver com o valor "S".
            if ($parametros["IDORIGBLOCOK"] == null && $parametros["IDGERABLOCOK"] == 'S') {
                throw new \Exception("Origem Dados (Bloco K) deve ser informado.");
            }
            // Verifica se o campo "Gerar Diferença Alíquota" possui algum valor informado.
            if ($parametros["IDGERDIFALIQ"] == null) {
                throw new \Exception("Gerar Diferença Alíquota deve ser informado.");
            }
            // Verifica se o campo "Versão" possui algum valor informado.
            if ($parametros["CDVERSAO"] == null) {
                throw new \Exception("Versão deve ser informada.");
            }
            // Validações adicionais caso o registro 1400 do campo
            // "Obrigatoriedade de Registros - Bloco 1" esteja marcado.
            if (substr($parametros["CDOBRIGAREGBL1"], 4, 1) == 'S') {
                // Busca o estado da inscrição estadual relacionada.
                $sgestado = $this->spedFactory->getEstadoInscricaoEstadual($params);
    
                // Validações para a tela exibida no caso do registro 1400 estar
                // marcado, somente para os estados "ES", "MG" e "SP".
                if ($sgestado == "ES" || $sgestado == "MG" || $sgestado == "SP") {
                    // Verifica se o campo "Livro Magnético" possui algum valor informado.
                    if ($row["CDLIVROMAGADIC"] == null) {
                        throw new \Exception("Livro Magnético deve ser informado.");
                    }
                    // Verifica se o campo "Data Inicial" possui algum valor informado.
                    // Obrigatório somente para o estado "MG".
                    if ($row["DTINIVAF"] == null && $sgestado == "MG") {
                        throw new \Exception("Data Inicial deve ser informada.");
                    }
                    // Verifica se o campo "Data Final" possui algum valor informado.
                    // Obrigatório somente para o estado "MG".
                    if ($row["DTFINVAF"] == null && $sgestado == "MG") {
                        throw new \Exception("Data Final deve ser informada.");
                    }
                }
            }
        }

        return $parametros;
    }

    /*
     * Função para validação dos dados informados para o SPED.
     */
    private function validateSped($row) {
        $params = array(
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "NRORG"         => $row["NRORG"]
        );
        
        if(isset($row['NRAGENDAMENTO'])){
            
            // Verifica se o campo "Arquivo" possui algum valor informado.
            if (empty($row["CDARQUIVO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Arquivo deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Arquivo deve ser informado.");
            }
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Livro Magnético" possui algum valor informado.
            if (empty($row["CDLIVROMAG"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Livro Magnético deve ser informado.");
            }
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Referência deve ser informada.");
            }
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Nome do Arquivo deve ser informado.");
            }
    
            if ($row["CDARQUIVO"] == "ICMS") {
                // Verifica se o campo "Inscrição Estadual" possui algum valor informado.
                if (empty($row["CDINSCESTA"])) {
                    $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                    $this->geraArquivo = false;
                    //throw new \Exception("Inscrição Estadual deve ser informada.");
                }
                // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
                if (empty($row["IDGERABLOCOH"])) {
                    $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                    $this->geraArquivo = false;
                    //throw new \Exception("Nome do Arquivo deve ser informado.");
                }
    
                if ($row["IDGERABLOCOH"] == 'S') {
                    // Verifica se o campo "Data Inicial Inventário" possui algum valor informado.
                    if (empty($row["DTINIPERFISC"])) {
                        $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                        $this->geraArquivo = false;
                        //throw new \Exception("Data Inicial Inventário deve ser informado.");
                    }
                    // Verifica se o campo "Data Final Inventário" possui algum valor informado.
                    if (empty($row["DTFINPERFISC"])) {
                        $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                        $this->geraArquivo = false;
                        //throw new \Exception("Data Final Inventário deve ser informado.");
                    }
                    // Verifica se o campo "Motivo do Inventário" possui algum valor informado.
                    if (empty($row["CDMOTINVENTARIO"])) {
                        $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                        $this->geraArquivo = false;
                        //throw new \Exception("Motivo do Inventário deve ser informado.");
                    }
                }
            }
            
            $sgestado = $this->spedFactory->getEstadoInscricaoEstadual($params);
            
            if ($sgestado == "DF"){
                if (empty($row["CDINSCMUNI"])) {
                    $this->mensagemError = $this->mensagemError.'<br />'.'Inscrição Municipal deve ser informada.';
                    $this->geraArquivo = false;
                    //throw new \Exception("Inscrição Municipal deve ser informado.");
                }
            }
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            //var_dump($row["NRORG"]);
            if(!isset($row['NRAGENDAMENTO'])){
                $nrorg = $this->environment->getNrOrgTrab();
                $row["NRORG"] = $nrorg;
            }
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
           
            // Seta a variável de impostos verificados no mapa conforme SPED.
            if ($row["CDARQUIVO"] == "ICMS") {
                // No caso do SPED Fiscal, esses são os impostos validados.
                $impostos = array('00','10','80');
            }
            else {
                // No caso do SPED Contribuição, esses são os impostos validados.
                $impostos = array('50', '60', '55', '65');
            }
            
            //var_dump($row);
            // Valida o mapa de apuração.
            $mapaApuracIsValid = $this->obrigacoesService->mapaApuracIsValid($row, $impostos);
    
            // Verifica se o mapa está inválido, lançando um erro em caso positivo.
            if (!$mapaApuracIsValid) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade do Arquivo deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Todos os mapas devem estar bloqueados para geração do arquivo.");
            }
        }
        else {    
            // Verifica se o campo "Arquivo" possui algum valor informado.
            if (empty($row["CDARQUIVO"])) {
                throw new \Exception("Arquivo deve ser informado.");
            }
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Livro Magnético" possui algum valor informado.
            if (empty($row["CDLIVROMAG"])) {
                throw new \Exception("Livro Magnético deve ser informado.");
            }
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                throw new \Exception("Referência deve ser informada.");
            }
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                throw new \Exception("Nome do Arquivo deve ser informado.");
            }
    
            if ($row["CDARQUIVO"] == "ICMS") {
                // Verifica se o campo "Inscrição Estadual" possui algum valor informado.
                if (empty($row["CDINSCESTA"])) {
                    throw new \Exception("Inscrição Estadual deve ser informada.");
                }
                // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
                if (empty($row["IDGERABLOCOH"])) {
                    throw new \Exception("Nome do Arquivo deve ser informado.");
                }
    
                if ($row["IDGERABLOCOH"] == 'S') {
                    // Verifica se o campo "Data Inicial Inventário" possui algum valor informado.
                    if (empty($row["DTINIPERFISC"])) {
                        throw new \Exception("Data Inicial Inventário deve ser informado.");
                    }
                    // Verifica se o campo "Data Final Inventário" possui algum valor informado.
                    if (empty($row["DTFINPERFISC"])) {
                        throw new \Exception("Data Final Inventário deve ser informado.");
                    }
                    // Verifica se o campo "Motivo do Inventário" possui algum valor informado.
                    if (empty($row["CDMOTINVENTARIO"])) {
                        throw new \Exception("Motivo do Inventário deve ser informado.");
                    }
                }
            }
    
            $sgestado = $this->spedFactory->getEstadoInscricaoEstadual($params);
            
            if ($sgestado == "DF"){
                if (empty($row["CDINSCMUNI"])) {
                    $this->mensagemError = $this->mensagemError.'<br />'.'Inscrição Municipal deve ser informada.';
                    $this->geraArquivo = false;
                    //throw new \Exception("Inscrição Municipal deve ser informado.");
                }
            }
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            //var_dump($row["NRORG"]);
            if(!isset($row['NRAGENDAMENTO'])){
                $nrorg = $this->environment->getNrOrgTrab();
                $row["NRORG"] = $nrorg;
            }
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
           
            // Seta a variável de impostos verificados no mapa conforme SPED.
            if ($row["CDARQUIVO"] == "ICMS") {
                // No caso do SPED Fiscal, esses são os impostos validados.
                $impostos = array('00','10','80');
            }
            else {
                // No caso do SPED Contribuição, esses são os impostos validados.
                $impostos = array('50', '60', '55', '65');
            }
            
            //var_dump($row);
            // Valida o mapa de apuração.
            $mapaApuracIsValid = $this->obrigacoesService->mapaApuracIsValid($row, $impostos);
    
            // Verifica se o mapa está inválido, lançando um erro em caso positivo.
            if (!$mapaApuracIsValid) {
                throw new \Exception("Todos os mapas devem estar bloqueados para geração do arquivo.");
            }
        }
    }
    
    public function findAgendamento() {
        // Busca os parâmetros conforme empresa e nrorg.
        $agendamento = $this->spedFactory->findAgendamento();

        return $agendamento;
    }
    
    public function findAgendamentoArquivo() {
        // Busca os parâmetros conforme empresa e nrorg.
        $agendamento = $this->spedFactory->findAgendamento();

        return $agendamento;
    }
    
    /*
     * Função para salvar um agendamento para exportação do SPED.
     */
    public function saveAgendamentoSped($row) {
        try {
            
            // Se o arquivo for "ICMS", redireciona para o método de exportação Fiscal.
            if ($row["CDARQUIVO"] == 'ICMS') {
                //$fileName = $this->exportarSpedFiscal($row);
                $row['CDTELA'] = 'FIS85000';
                $tipoSped = 'SPED Fiscal ICMS/IPI';
                
            }
            // Se o arquivo for "ICMS", redireciona para o método de exportação Contribuições.
            else if ($row["CDARQUIVO"] == 'CONT') {
                //$fileName = $this->exportarSpedContribuicoes($row);
                $row['CDTELA'] = 'FIS95000';
                $tipoSped = 'SPED Contribuições';
            }
            // Caso não seja nenhum destes, informa que o arquivo escolhido é inválido.
            else {
                throw new \Exception("Arquivo inválido.");
            }
            
            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtinicial = $dtinicial->format("d/m/Y");
            $dtfinal = $dtfinal->format("d/m/Y");
    
            $row["DTINICIAL"] = $dtinicial;
            $row["DTFINAL"] = $dtfinal;
            
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $row["NRAGENDAMENTO"] = $this->newCode->getCodeByTableName('NRAGENDAMENTO', $row["NRORG"], 1, 10);
            }
            
            if ($row["CDARQUIVO"] == 'ICMS') {
                if($row['CDINSCESTA'] == 'T'){
                    $arrayInscEsta = $this->spedFactory->getSpedFiscalInscEsta($row);
                    $cdinscesta = '_';
                    
                    foreach($arrayInscEsta as $value){
                        $cdinscesta = $cdinscesta.$value['CDINSCESTA'].'_';
                    }
                    
                    $row['CDINSCESTA'] = $cdinscesta;
                } else if(is_array($row['CDINSCESTA']) == true){
                    $arrayInscEsta = $row['CDINSCESTA'];
                    $cdinscesta = '_';
                    
                    foreach($arrayInscEsta as $value){
                        $cdinscesta = $cdinscesta.$value.'_';
                    }
                    
                    $row['CDINSCESTA'] = $cdinscesta;
                } 
            } else if($row["CDARQUIVO"] == 'CONT') {
                if($row['CDEMPRESA'] == 'T'){
                    $arrayEmpresa = $this->spedFactory->getSpedContEmpresa();
                    $cdempresa = '_';
                    
                    foreach($arrayEmpresa as $value){
                        $cdempresa = $cdempresa.$value['CDEMPRESA'].'_';
                    }
                    
                    $row['CDEMPRESA'] = $cdempresa;
                } else if(is_array($row['CDEMPRESA']) == true){
                    $arrayEmpresa = $row['CDEMPRESA'];
                    $cdempresa = '_';
                    
                    foreach($arrayEmpresa as $value){
                        $cdempresa = $cdempresa.$value.'_';
                    }
                    
                    $row['CDEMPRESA'] = $cdempresa;
                }
            }
            
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $this->spedFactory->insertAgendamentoSped($row);
            } else {
              
                $this->spedFactory->updateAgendamentoSped($row);
                
                $arquivos = $this->spedFactory->getAgendamentoArquivoSped($row);
                
                if($arquivos){
                    $getConfigSMTP = $this->spedFactory->getConfiguraSMTP($row)[0];
                    
                    if(isset($getConfigSMTP['CALLSERVPATH'])){
                        foreach($arquivos as $arquivo){
                            if(file_exists($arquivo['NMPATHARQ'])) {
                                unlink($arquivo['NMPATHARQ']);
                            }
                        }            
                    } else {
                        foreach($arquivos as $arquivo){
                            $this->fileUploader->deleteFile( $arquivo['NMPATHARQ']);
                        }
                    }
                    $this->spedFactory->deleteAgendamentoArquivoSped($arquivo);
                }
                
                $idstatus = 'P';
                $this->spedFactory->updateAgendamentoIdStatus($row, $idstatus);
                
            }
            
            $agendamento = $this->spedFactory->getAgendamentoSpedInsert($row);
            
            if($agendamento){
                
                $agendamento['NRORG'] = $row['NRORG'];
                $getConfigSMTP = $this->spedFactory->getConfiguraSMTP($agendamento)[0];
                if(isset($row['__is_new']) && $row['__is_new'] == true){
                    $nomeJob = $row['CDTELA'].'_'.$row['NRAGENDAMENTO'];
                } else {
                    $nomeJob = $row['CDTELA'].'_'.str_pad($row['NRAGENDAMENTO'], 10, "0", STR_PAD_LEFT);
                }
                $dataContabAuto = $row['DTAGENDAMENTO'];
                $sequencialJob = $row['NRAGENDAMENTO'];
                $nrorg = $row['NRORG'];
                $descricao = $row['CDTELA'].' - '.$tipoSped.'. Exportação das declarações Acessórias de forma automática na data de '.$row['DTAGENDAMENTO'];
                //Conferir caminho do diretorio do xampp
                $arqCallService = $getConfigSMTP['CALLSERVPATH'];//'C:\Teknisa\CallService.php';
                //$dirphp = str_replace('/','\',$DIRPHP);
                $dirphp = $getConfigSMTP['DIRPHP'];//'C:/xampp/php';
                $jenkinsPath = $getConfigSMTP['JENKINSPATH'];//'http://192.168.120.132:8080/';
                $execPath = '/execAgendamento';
                $periodicidade = $row['IDPERIAGEN'];
                $jenkinsCreateItem = $getConfigSMTP['JENKCREATITEM'];
                $this->agendamento->criaJob($nomeJob, $dataContabAuto, $sequencialJob, $nrorg, $descricao, $arqCallService, $dirphp, $jenkinsPath, $execPath, $periodicidade, $jenkinsCreateItem);
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
    
    /*
     * Função para executar um agendamento para exportação os arquivos do SPED, chamado pelo Jenkins.
     */
    public function execAgendamento($row){
        try{
            $agendamento = $this->spedFactory->findAgendamentoExec($row)[0];
            
            if($agendamento){
                if ($agendamento['CDTELA'] == 'FIS85000'){
                    $this->mensagemBody = '';
                    
                    $agendamento["CDARQUIVO"] = 'ICMS';
                    $agendamento["NRORG"] = $row['NRORG'];
                    
                    $agendamento["DTREFINI"] = explode(' ', $agendamento["DTREFINI"])[0];
                    
                    $agendamento["DTEXECAGEND"] = $agendamento["DTATUAL"];
                    
                    //Necessário Fazer a verificação
                    $agendamento["CDLIVROMAGADIC"] = null;
                    $agendamento["DTINIVAF"] = null;
                    $agendamento["DTFINVAF"] = null;
                    
                    $agendamento['CDINSCESTAARRAY'] = $agendamento['CDINSCESTA'];
                    $arrayInscEsta = explode('_',$agendamento['CDINSCESTAARRAY']);
                    
                    $getConfigSMTP = $this->spedFactory->getConfiguraSMTP($agendamento)[0];
                    $getContatos = $this->spedFactory->getContatos($agendamento);
                    
                    $mail = new PHPMailer;
                    $mail->CharSet = 'UTF-8';
                    $mail->isSMTP();                        // Set mailer to use SMTP
                    $mail->isHTML();
                    $mail->Host = $getConfigSMTP['SERVIDORSMTP'];       // Specify main and backup SMTP servers
                    $mail->SMTPAuth = true;                 // Enable SMTP authentication
                    $mail->Username = $getConfigSMTP['LOGINSMTP'];    // SMTP username
                    $mail->Password = $getConfigSMTP['SENHASMTP'];    // SMTP password
                    $mail->From = $getConfigSMTP['LOGINSMTP'];
                    $mail->FromName = 'ERP Pebbian';
                    $mail->Subject = 'Sped Fiscal de '.$agendamento['DTAGENDAMENTO'];
                    // $mail->addAddress($cliente['DSEMAILCLIE']);
                    // $mail->addAddress($vendedor['DSEMAILOPER']);
                    
                    foreach($getContatos as $contato){
                        $mail->addAddress($contato['EMAIL']);
                    };
                    
                    $agendamento["NMEMPRESA"] = $this->spedFactory->getNmEmpresa($agendamento)["NMEMPRESA"];
                            
                    $this->mensagemBody = 'Caro cliente, '.'<br />'.'<br />';
                    $this->mensagemBodySucess = 'Arquivo de Sped Fiscal gerado com Sucesso!'
                    .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                    $this->mensagemErrorBody = 'Erro ao gerar o arquivo de Sped Contribuição:'.'<br />'.'<br />'
                    .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                    
                    
                    $contErro = 0;
                    $contSucess = 0;
                    
                    foreach( $arrayInscEsta as $inscEsta ){
                        if(isset($inscEsta) && $inscEsta != ''){
                            
                            $dataName = str_replace('/', '', explode(' ', $agendamento["DTATUAL"])[0]);
                            $horaName = str_replace(':', '', explode(' ', $agendamento["DTATUAL"])[1]);
                            
                            if($agendamento['IDPERIAGEN'] == '0'){
                                $dtinicial = $agendamento["DTINICIAL"];
                                $dtfinal = $agendamento["DTFINAL"];
                            } 
                            else if($agendamento['IDPERIAGEN'] == '1'){
                                $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                                $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                                $dtinicial = $dtinicial->format("d/m/Y");
                                $dtfinal = $dtfinal->format("d/m/Y");
                            } 
                            else if($agendamento['IDPERIAGEN'] == '2'){
                                $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                                $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                                $dtinicial = $dtinicial->format("d/m/Y");
                                $dtfinal = $dtfinal->format("d/m/Y");
                            }
                            
                            $agendamento["DTINICIAL"] = $dtinicial;
                            $agendamento["DTFINAL"] = $dtfinal;
                            
                            $agendamento['CDINSCESTA'] = $inscEsta;
                            $agendamento["NMARQUIVO"] = $agendamento['CDTELA'].$agendamento['NRAGENDAMENTO'].'_'.$inscEsta.'_'.$dataName.$horaName;
                              
                            $fileName = $this->exportarSped($agendamento);
                            
                            if($this->mensagemError == ''){
                                $this->mensagemBodySucess = $this->mensagemBodySucess
                                .'<br />'.'Inscrição Estadual: '.$agendamento['CDINSCESTA'];
                                ++$contSucess;
                                
                            } else {
                                $this->mensagemErrorBody = $this->mensagemErrorBody
                                .'<br />'.'Inscrição Estadual: '.$agendamento['CDINSCESTA']
                                .$this->mensagemError.'<br />'.'<br />';
                                ++$contErro;
                            }
                            
                            if($fileName){
                                $file = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName;
                                
                                if(isset($getConfigSMTP['CALLSERVPATH'])){
                                    $filesDir = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'files' . DIRECTORY_SEPARATOR . 'SPED';
                                    
                                    if (!file_exists($filesDir)) {
                                        mkdir($filesDir , 0777, true);
                                    }
                                    
                                    rename($file, $filesDir. DIRECTORY_SEPARATOR . $fileName);

                                    $agendamento["NMPATHARQ"] = $filesDir. DIRECTORY_SEPARATOR . $fileName;
                                } else {
                                    $filePath = $this->saveInFileUplouder($file, $fileName);
                                    $agendamento["NMPATHARQ"] = $filePath;
                                    unlink($file);
                                }
                                
                                $verifArq = $this->spedFactory->verAgendamentoArquivoSped($agendamento);
                                
                                if(empty($verifArq)){
                                    $agendamento["NRARQUIVO"] = $this->newCode->getCodeByTableName('NRARQUIVO', $agendamento["NRORG"], 1, 10);
                                    $this->spedFactory->insertAgendamentoArquivoSped($agendamento);
							    }
                            }
                    
                        }
                    }
                    
                    if(($contErro > 0) && ($contSucess > 0)){
                        $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'.'<br />'
                        .'---------------------------------------------------------------------'.'<br />'
                        .$this->mensagemErrorBody
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    } 
                    else if($contErro > 0){
                        $mail->Body = $this->mensagemBody.'<br />'
                        .$this->mensagemErrorBody
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    }
                    else if($contSucess > 0){
                        $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    }
                    
                }
                // Se o arquivo for "ICMS", redireciona para o método de exportação Contribuições.
                else if ($agendamento['CDTELA'] == 'FIS95000') {
                    
                    $this->mensagemBody = '';
                    
                    $agendamento["CDARQUIVO"] = 'CONT';
                    $agendamento["NMARQUIVO"] = $agendamento['CDTELA'].$agendamento['NRAGENDAMENTO'];
                    
                    $agendamento["NRORG"] = $row['NRORG'];
                    
                    $agendamento["DTEXECAGEND"] = $agendamento["DTATUAL"];
                
                    $agendamento["DTREFINI"] = explode(' ', $agendamento["DTREFINI"])[0];
                    
                    //Necessário Fazer a verificação
                    $agendamento["CDLIVROMAGADIC"] = null;
                    $agendamento["DTINIVAF"] = null;
                    $agendamento["DTFINVAF"] = null;
                    
                    $agendamento['CDEMPRESAARRAY'] = $agendamento['CDEMPRESA'];
                    $arrayEmpresa = explode('_',$agendamento['CDEMPRESAARRAY']);
                    
                    $getConfigSMTP = $this->spedFactory->getConfiguraSMTP($agendamento)[0];
                    $getContatos = $this->spedFactory->getContatos($agendamento);
                    
                    $mail = new PHPMailer;
                    $mail->CharSet = 'UTF-8';
                    $mail->isSMTP();                        // Set mailer to use SMTP
                    $mail->isHTML();
                    $mail->Host = $getConfigSMTP['SERVIDORSMTP'];       // Specify main and backup SMTP servers
                    $mail->SMTPAuth = true;                             // Enable SMTP authentication
                    $mail->Username = $getConfigSMTP['LOGINSMTP'];      // SMTP username
                    $mail->Password = $getConfigSMTP['SENHASMTP'];      // SMTP password
                    $mail->From = $getConfigSMTP['LOGINSMTP'];
                    $mail->FromName = 'ERP Pebbian';
                    $mail->Subject = 'Sped Contribuições de '.$agendamento['DTAGENDAMENTO'];
                    // $mail->addAddress($cliente['DSEMAILCLIE']);
                    // $mail->addAddress($vendedor['DSEMAILOPER']);
                    
                    // Mudar para pegar o email do operador logado.
                    
                    foreach($getContatos as $contato){
                        $mail->addAddress($contato['EMAIL']);
                    };
            
                    $this->mensagemBody = 'Caro cliente, '.'<br />'.'<br />';
                    $this->mensagemBodySucess = 'Arquivo de Sped Contribuição gerado com Sucesso!';
                    $this->mensagemErrorBody = '<br />'.'<br />'.'Erro ao gerar o arquivo de Sped Contribuição:'.'<br />'.'<br />';
                    
                    $contErro = 0;
                    $contSucess = 0;
                    
                    foreach( $arrayEmpresa as $empresa ){
                        if(isset($empresa) && $empresa != ''){
                            
                            $this->geraArquivo = true;
                            
                            $dataName = str_replace('/', '', explode(' ', $agendamento["DTATUAL"])[0]);
                            $horaName = str_replace(':', '', explode(' ', $agendamento["DTATUAL"])[1]);
                            
                            if($agendamento['IDPERIAGEN'] === '0'){
                                $dtinicial = $agendamento["DTINICIAL"];
                                $dtfinal = $agendamento["DTFINAL"];
                            } 
                            else if($agendamento['IDPERIAGEN'] === '1'){
                                $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                                $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                                $dtinicial = $dtinicial->format("d/m/Y");
                                $dtfinal = $dtfinal->format("d/m/Y");
                            } 
                            else if($agendamento['IDPERIAGEN'] === '2'){
                                $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                                $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                                $dtinicial = $dtinicial->format("d/m/Y");
                                $dtfinal = $dtfinal->format("d/m/Y");
                            }
                            
                            $agendamento["DTINICIAL"] = $dtinicial;
                            $agendamento["DTFINAL"] = $dtfinal;
                            
                            $agendamento['CDEMPRESA'] = $empresa;
                            $agendamento["NMARQUIVO"] = $agendamento['CDTELA'].$agendamento['NRAGENDAMENTO'].'_'.$empresa.'_'.$dataName.$horaName;
                              
                            $fileName = $this->exportarSped($agendamento);
                            
                            $agendamento["NMEMPRESA"] = $this->spedFactory->getNmEmpresa($agendamento)["NMEMPRESA"];
                            
                            if($this->mensagemError == ''){
                                $this->mensagemBodySucess = $this->mensagemBodySucess
                                .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                                ++$contSucess;
                                
                            } else {
                                $this->mensagemErrorBody = $this->mensagemErrorBody. 
                                '<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA']
                                .$this->mensagemError.'<br />'.'<br />';
                                ++$contErro;
                            }
            
                            if($fileName){
                                $file = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName;
                                
                                if(isset($getConfigSMTP['CALLSERVPATH'])){
                                    $filesDir = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'files' . DIRECTORY_SEPARATOR . 'SPED';
                                    // var_dump(dirname(__DIR__) . DIRECTORY_SEPARATOR . 'files');die;
                                    
                                    if (!file_exists($filesDir)) {
                                        mkdir($filesDir , 0777, true);
                                    }
                                    
                                    rename($file, $filesDir. DIRECTORY_SEPARATOR . $fileName);

                                    $agendamento["NMPATHARQ"] = $filesDir. DIRECTORY_SEPARATOR . $fileName ;
                                } else {
                                    $filePath = $this->saveInFileUplouder($file, $fileName);
                                    $agendamento["NMPATHARQ"] = $filePath;
                                    unlink($file);
                                }
                                
                                $verifArq = $this->spedFactory->verAgendamentoArquivoSped($agendamento);
                                
                                if(empty($verifArq)){
                                    $agendamento["NRARQUIVO"] = $this->newCode->getCodeByTableName('NRARQUIVO', $agendamento["NRORG"], 1, 10);
                                    $this->spedFactory->insertAgendamentoArquivoSped($agendamento);
                                }
                                
                            }
                    
                        }
                        
                    }
                    
                    if(($contErro > 0) && ($contSucess > 0)){
                        $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'.'<br />'
                        .'---------------------------------------------------------------------'.'<br />'
                        .$this->mensagemErrorBody.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    } 
                    else if($contErro > 0){
                        $mail->Body = $this->mensagemBody.'<br />'
                        .$this->mensagemErrorBody.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    }
                    else if($contSucess > 0){
                        $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    }
                    
                }
                
                $idstatus = 'R';
                $this->spedFactory->updateAgendamentoIdStatus($agendamento, $idstatus);
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
    
    /*
     * Função para deletar um agendamento para exportação do SPED.
     */
    public function deleteAgendamento($row){
        try {    
            
            $agendamento = $this->spedFactory->getAgendamentoSpedInsert($row)[0];
            
            if($agendamento){
                $nomeJob = $agendamento['CDTELA'].'_'.str_pad($agendamento['NRAGENDAMENTO'], 10, "0", STR_PAD_LEFT);
                $getConfigSMTP = $this->spedFactory->getConfiguraSMTP($agendamento)[0];
                
                $this->agendamento->apagaJob($nomeJob, null, $getConfigSMTP['JENKINSPATH'], $getConfigSMTP['CALLSERVPATH']);
                
                $arquivos = $this->spedFactory->getAgendamentoArquivoSped($agendamento);
                
                if($arquivos){
                    
                    foreach($arquivos as $arquivo){
                        if(file_exists($arquivo['NMPATHARQ'])){
                            unlink($arquivo['NMPATHARQ']);
                        }
                    }
                    
                    $this->spedFactory->deleteAgendamentoArquivoSped($arquivo);
                    $this->spedFactory->deleteAgendamentoSped($arquivo);
                    
                } else {
                    $this->spedFactory->deleteAgendamentoSped($agendamento);
                }
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
                
    }
    
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function saveInFileUplouder($tmpFile , $fileName) {
        
        if (file_exists($tmpFile)) {
            $file = file_get_contents($tmpFile);
            $file = base64_encode($file);
        
            $file = 'data: '.mime_content_type($tmpFile).';base64,'.$file;
            
            $lastModify = filectime($tmpFile);
        
            $file = array(
                "name" => $fileName, 
                "lastModified" => $lastModify,
                "lastModifiedDate" => '',
                "webkitRelativePath" => '',
                "type" => '',
                "size"  => filesize($tmpFile),
                "b64File" => $file);
                
            $row['FILE']    = array();
            $row['FILE'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'FILE', 'ObrigacoesAcessorias');
            
            return $row['FILE'][0]['path']; 
            /*
            
            /*$fileContent = file_get_contents($tmpFile);
            $fileBase64 = base64_encode($fileContent);
        
            $fileBase64Zeedhi = 'data: '.mime_content_type($tmpFile).';base64,'.$fileBase64;
            
            $row = [
                'FILE' => [
                    [
                        "name" => $fileName.'.txt', 
                        "type" => 'application/txt', 
                        "size" => strlen($fileContent), 
                        "path" => mime_content_type($tmpFile), 
                        "b64File" => $fileBase64Zeedhi
                    ]
                ],
                'FOLDER' => 'ObrigacoesAcessorias'
            ];
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */
            
        } else { 
            return false;
        }
    }
    
    public function deleteArquivos($row){
        try{
            
            if (strpos($row['NMPATHARQ'], '/tmp/') !== false) {
                if(isset($row['NMPATHARQ']) && file_exists($row['NMPATHARQ'])){
                    unlink($row['NMPATHARQ']);
                    $this->spedFactory->deleteArquivo($row);
                }
            } else {
                $this->fileUploader->deleteFile($row['NMPATHARQ']);
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
    
    public function getFileServerUrl(){
            return $this->_fileServerUrl;
    }
    
}